@@ -14,91 +14,91 @@ C Client Library for memcached (libmemcached, -lmemcached)
 
   memcached_return
     memcached_set (memcached_st *ptr,
-                   char *key, size_t key_length, 
-                   char *value, size_t value_length, 
+                   const char *key, size_t key_length, 
+                   const char *value, size_t value_length, 
                    time_t expiration,
                    uint32_t flags);
 
   memcached_return
     memcached_add (memcached_st *ptr,
-                   char *key, size_t key_length,
-                   char *value, size_t value_length, 
+                   const char *key, size_t key_length,
+                   const char *value, size_t value_length, 
                    time_t expiration,
                    uint32_t flags);
 
   memcached_return
     memcached_replace (memcached_st *ptr,
-                       char *key, size_t key_length,
-                       char *value, size_t value_length, 
+                       const char *key, size_t key_length,
+                       const char *value, size_t value_length, 
                        time_t expiration,
                        uint32_t flags);
 
   memcached_return 
     memcached_prepend(memcached_st *ptr, 
-                      char *key, size_t key_length,
-                      char *value, size_t value_length, 
+                      const char *key, size_t key_length,
+                      const char *value, size_t value_length, 
                       time_t expiration,
                       uint32_t flags)
 
   memcached_return 
     memcached_append(memcached_st *ptr, 
-                     char *key, size_t key_length,
-                      char *value, size_t value_length, 
+                     const char *key, size_t key_length,
+                      const char *value, size_t value_length, 
                       time_t expiration,
                       uint32_t flags)
   memcached_return 
     memcached_cas(memcached_st *ptr, 
-                  char *key, size_t key_length,
-                  char *value, size_t value_length, 
+                  const char *key, size_t key_length,
+                  const char *value, size_t value_length, 
                   time_t expiration,
                   uint32_t flags,
                   uint64_t cas);
 
   memcached_return 
     memcached_set_by_key(memcached_st *ptr, 
-                         char *master_key, size_t master_key_length, 
-                         char *key, size_t key_length, 
-                         char *value, size_t value_length, 
+                         const char *master_key, size_t master_key_length, 
+                         const char *key, size_t key_length, 
+                         const char *value, size_t value_length, 
                          time_t expiration,
                          uint32_t flags);
 
   memcached_return 
     memcached_add_by_key(memcached_st *ptr, 
-                         char *master_key, size_t master_key_length,
-                         char *key, size_t key_length,
-                         char *value, size_t value_length, 
+                         const char *master_key, size_t master_key_length,
+                         const char *key, size_t key_length,
+                         const char *value, size_t value_length, 
                          time_t expiration,
                          uint32_t flags);
 
   memcached_return 
     memcached_replace_by_key(memcached_st *ptr, 
-                             char *master_key, size_t master_key_length,
-                             char *key, size_t key_length,
-                             char *value, size_t value_length, 
+                             const char *master_key, size_t master_key_length,
+                             const char *key, size_t key_length,
+                             const char *value, size_t value_length, 
                              time_t expiration,
                              uint32_t flags);
 
   memcached_return 
     memcached_prepend_by_key(memcached_st *ptr, 
-                             char *master_key, size_t master_key_length,
-                             char *key, size_t key_length,
-                             char *value, size_t value_length, 
+                             const char *master_key, size_t master_key_length,
+                             const char *key, size_t key_length,
+                             const char *value, size_t value_length, 
                              time_t expiration,
                              uint32_t flags);
 
   memcached_return 
     memcached_append_by_key(memcached_st *ptr, 
-                            char *master_key, size_t master_key_length,
-                            char *key, size_t key_length,
-                            char *value, size_t value_length, 
+                            const char *master_key, size_t master_key_length,
+                            const char *key, size_t key_length,
+                            const char *value, size_t value_length, 
                             time_t expiration,
                             uint32_t flags);
 
   memcached_return 
     memcached_cas_by_key(memcached_st *ptr, 
-                         char *master_key, size_t master_key_length,
-                         char *key, size_t key_length,
-                         char *value, size_t value_length, 
+                         const char *master_key, size_t master_key_length,
+                         const char *key, size_t key_length,
+                         const char *value, size_t value_length, 
                          time_t expiration,
                          uint32_t flags,
                          uint64_t cas);
