@@ -16,7 +16,7 @@ typedef enum {
   TEST_SUCCESS= 0, /* Backwards compatibility */
   TEST_FAILURE,
   TEST_MEMORY_ALLOCATION_FAILURE,
-  TEST_MAXIMUM_RETURN, /* Always add new error code before */
+  TEST_MAXIMUM_RETURN /* Always add new error code before */
 } test_return;
 
 struct test_st {
